library ac_dart.example.router;

import 'package:atlassian_connect_shelf/server.dart';
import 'package:atlassian_connect_shelf/acdart_shelf_middleware.dart' as m;
import 'package:http_server/http_server.dart';
import 'package:logging/logging.dart';
import 'package:shelf/shelf.dart';
import 'package:shelf_route/shelf_route.dart' as route;
import 'dart:convert';

final Logger _log = new Logger('ac_dart.example.router.initRoutes');

const String _ISSUE_KEY = 'issueKey';

void initRoutes(AcDartMainRouteBuilder routeBuilder, String addonKey, 
                Uri addonBaseUri) {
  
  routeBuilder.addInstallationRoute();
  routeBuilder.addDescriptorTemplateRoute();
  
  routeBuilder.addHostServiceSessionTokenRoute(_fetchJiraIssue, 
      '/issue/{$_ISSUE_KEY}');

  // TODO: consider a directory server rather than file per file
  routeBuilder.addJwtRoute(
      m.moustacheFile('ui/exampleUI.html', includeExtraParams: true), 
      '/ui/exampleUI.html', method: 'GET');  
}

dynamic _fetchJiraIssue(AcHostService hostService, Request req) {
  final issueKey = route.getPathVariables(req)['$_ISSUE_KEY'];
  _log.fine('fetching jira issue $issueKey');

  // TODO: should be able to have the http client know about the session context
  // and automatically do this
  final sessionContext = m.getSessionContext(req);
  
  final jiraResponse = hostService.httpClientForHostKey(sessionContext.hostKey)
      .then((httpClient) => 
          httpClient.get('rest/api/2/issue/$issueKey', signRequest: true))
      .then((req) => req.close());

  return jiraResponse
      .then(HttpBodyHandler.processResponse)
      .then((body) {
    
        // TODO: encapsulate content type
        return new Response.ok(JSON.encode(body.body), 
            headers: { 'content-type': 'application/json' });
  });


}
